/**********************************************************
KPlayer for E2 v0.3
Plugin for FsMan - mplayer control with graphic gui

Written  by Strelok Vlad Kolerts
10/02/2009
Email: admin@mobsk.org.ua
ICQ: 497123239
***********************************************************/

#include <ZApplication.h>
#include <ZKbMainWidget.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <getopt.h>
#include <dirent.h>
#include <sys/stat.h>

#include <qfile.h>
#include <qdir.h>
#include <qdatastream.h>
#include <qtextstream.h>

#include <qcopchannel_qws.h>

#include <ZConfig.h>
#include <qtimer.h>

#include <KString.h>
#include "tool/mp3teg.h"

extern "C" int UTIL_GetIncomingCallStatus();


int mfd_in = -1;
//int mfd_out = -1;
int pid = 0;
bool playlist;

class KPlayer : public ZKbMainWidget
{
    Q_OBJECT

public:
    KPlayer(QString mfile);
    ~KPlayer();
	
	QCopChannel *cop;
	QCopChannel *cop_hardkey;
	
	QPainter	p;
	QPixmap* 	bg_buf;
	QPixmap*	about_scr;
	QPixmap 	p_bg;
	QPixmap 	p_play;
	QPixmap 	p_pause;
	QPixmap 	p_next;
	QPixmap 	p_prew;
	QPixmap 	p_vol;
	int			p_bg_height;
	int			p_vol_height;
	float		p_vol_step;
	int			pp_x;
	int			pp_y;
	int			next_x;
	int			next_y;
	int			prew_x;
	int			prew_y;
	int			vol_t_x;
	int			vol_t_y;
	int			vol_p_x;
	int			vol_p_y;
	int			kbps_x;
	int			kbps_y;
	int			khz_x;
	int			khz_y;
	int			chanels_x;
	int			chanels_y;
	int			s_name[4];
	int			name_font_size;
	int			vol_font_size;
	int			detal_font_size;
	
	QFont		name_font;
	QFont		vol_font;
	QFont		detal_font;
	
	QString		name_font_col;
	QString		vol_font_col;
	QString		detal_font_col;
	
	QColor		name_font_Col;
	QColor		vol_font_Col;
	QColor		detal_font_Col;
	
	QString path;
	QString mplayer;
	QString skin_path;
	QString work_folder;
	
	QStringList	playList;
	bool readTegs;
	QString tegArtist, tegTitle, tegAlbum, tegYear, tegGenre, tegComment, tegBitrate, tegHz, tegChanels;
	
	int paused;
	int prew_next;
	bool about;
	int about_c;
	
	int volume;
	bool mute;
	
	int paint;
	
	QString title, text2, text3, text4;
	
	QString		m_path, m_name;
	QTimer*		timer;
	int leight_name, pos_text, name_scrool;
	
    void readConfig();
	void readSkinConfig();
	//void PlayListGen();
	void readMP3tegs();
	void paintPlayer();
	void paintScrollText();
	void paintAbout();
	void drawAbout();
	
	bool ypoCkhc();
	
	//void nextTrack();
	
	void keyPressEvent(QKeyEvent *event);
	void keyReleaseEvent(QKeyEvent *event);
	void focusOutEvent(QFocusEvent* event);
	void focusInEvent(QFocusEvent* event);
	
	QString getPath ( const QString &AFileName );
	QString getFileName(const QString &AFileName);
	QString getMPlayerPath() { return mplayer; }
	QString getHomePath() { return path; }
	
	
public slots:
	void animate();
	void slotQCopReceived(const QCString &msg, const QByteArray &data);
	//void slotHardKeyQCopReceived(const QCString &msg, const QByteArray &data);
};

KPlayer::KPlayer(QString mfile) : 
    ZKbMainWidget(ZHeader::FULL_TYPE, NULL, "ZMainWidget", 0)
	{
		setBackgroundMode(NoBackground);
		setFullScreenMode(true, true);
		resize(240, 320);
		m_path=UTF(mfile);
		
		leight_name=0;pos_text=0;name_scrool=1;
		
		volume=16;
		paused=0;
		mute=false;
		about=false;
		about_c = 20;
		
		readTegs=true;
		tegBitrate="---";
		tegHz="-----";
		tegChanels="Unknown";
		
		readConfig();
		readSkinConfig();
		
		//PlayListGen();
		
		name_font=QFont(NULL, name_font_size, 25, false);
		vol_font=QFont(NULL, vol_font_size, 25, false);
		detal_font=QFont(NULL, detal_font_size, 25, false);
		qDebug("Skin font init ok");
		name_font_Col=QColor(atoi(name_font_col.left(3)),atoi(name_font_col.mid(4,3)),atoi(name_font_col.right(3)));
		vol_font_Col=QColor(atoi(vol_font_col.left(3)),atoi(vol_font_col.mid(4,3)),atoi(vol_font_col.right(3)));
		detal_font_Col=QColor(atoi(detal_font_col.left(3)),atoi(detal_font_col.mid(4,3)),atoi(detal_font_col.right(3)));
		qDebug("Skin color init ok");
		
		p_bg.load(skin_path+"bg.png");
		p_play.load(skin_path+"play.png");
		p_pause.load(skin_path+"pause.png");
		p_next.load(skin_path+"next.png");
		p_prew.load(skin_path+"prew.png");
		p_vol.load(skin_path+"volume.png");
		
		p_bg_height=p_bg.height();		
		bg_buf = new QPixmap(240, p_bg_height);
		
		p_vol_height=p_vol.height();
		p_vol_step=(float)p_vol.width()/100;
		
		about_scr = new QPixmap(220, 120);
		drawAbout();
		
		setFont(name_font);
		m_name=getFileName(m_path);
		
		cop = new QCopChannel("SVKPlayer", this);
		connect(cop, SIGNAL(received(const QCString&, const QByteArray&)), this, SLOT(slotQCopReceived(const QCString&, const QByteArray&)));
		
		cop_hardkey = new QCopChannel("/hardkey/bc", this);
		connect(cop_hardkey, SIGNAL(received(const QCString&, const QByteArray&)), this, SLOT(slotQCopReceived(const QCString&, const QByteArray&)));
		//connect(cop_hardkey, SIGNAL(received(const QCString&, const QByteArray&)), this, SLOT(slotQCopReceived(const QCString&)));
		//connect(cop_hardkey, SIGNAL(received(const QCString&, const QByteArray&)), this, SLOT(slotHardKeyQCopReceived(const QCString&, const QByteArray&)));
		
		timer = new QTimer(this);
		connect( timer, SIGNAL(timeout()), SLOT(animate()) );
		timer->start(40);
		//::system("pmtool -n -t 7200 -b 7200");
	}

KPlayer::~KPlayer()
	{
		//::system("pmtool -n -b 20 -t 120");
	}

void KPlayer::paintPlayer()
	{
		paint=1;
	///	
		p.begin( bg_buf ,this);
		
		p.setBrush(Qt::black);
		
		p.drawPixmap(0, 0, p_bg);
	///	
		if(paused)	p.drawPixmap(pp_x, pp_y, p_pause);
		else		p.drawPixmap(pp_x, pp_y, p_play);
	//	
		if(prew_next==1)	p.drawPixmap(next_x, next_y, p_next);
		if(prew_next==2)	p.drawPixmap(prew_x, prew_y, p_prew);
		
		//p.drawText(0, 20, QString("mfd_in=%1").arg(mfd_in), -1, QPainter::RTL);
	///
		if(!mute && volume>0)
		{
			p.setClipRect(vol_p_x, vol_p_y, (int)(p_vol_step*volume), p_vol_height);
			p.drawPixmap(vol_p_x, vol_p_y, p_vol);
			p.setClipRect(0, 0, 240, p_bg_height);
		}
	//
		p.setFont(vol_font);
		p.setPen(vol_font_Col);
		
		if(volume<1 or mute)
		{
			p.drawText(vol_t_x, vol_t_y+vol_font_size, QString("off"), -1, QPainter::RTL);
		}
		else	p.drawText(vol_t_x, vol_t_y+vol_font_size, QString("%1").arg(volume), -1, QPainter::RTL);		
	///
		p.setFont(detal_font);
		p.setPen(detal_font_Col);
		p.drawText(kbps_x, kbps_y+detal_font_size, tegBitrate, -1, QPainter::RTL);
		p.drawText(khz_x, khz_y+detal_font_size, tegHz, -1, QPainter::RTL);
		p.drawText(chanels_x, chanels_y+detal_font_size, tegChanels, -1, QPainter::RTL);
	///	
		p.end();
		bitBlt( this, 0, 0, bg_buf );
	}

void KPlayer::paintScrollText()
	{
	///	
		p.begin( bg_buf ,this);
		
		p.setPen(name_font_Col);
		p.setFont(name_font);
		p.setClipRect(s_name[0], s_name[1], s_name[2], s_name[3]);
	//
		p.drawPixmap(0, 0, p_bg);
	//
		//p.drawText(5, 35, info[1], -1, QPainter::RTL);
		if(leight_name>s_name[2]) 
		{
			p.drawText(s_name[0]-pos_text, s_name[1]+name_font_size, m_name, -1, QPainter::RTL);
			pos_text=pos_text+name_scrool;
			if ( leight_name-pos_text==s_name[2]-2 or pos_text==-2)	name_scrool=-name_scrool;
		}
		else
			p.drawText(s_name[0], s_name[1], s_name[2], s_name[3], Qt::AlignCenter, m_name, -1, 0);
		p.end();
	///	
		bitBlt( this, 0, 0, bg_buf );
	}

	
void KPlayer::paintAbout()
	{
		bitBlt( this, 10, 100, about_scr );
	}
/*
void KPlayer::nextTrack()
	{
		QString text=getFileName(m_path);
		QStringList::ConstIterator it = playList.begin();
		bool save=false;bool end=false;
		
		while (!end)
		{
			if(*it!="." && *it!="..")
			{
				qDebug("PL_FIND::"+*it);
				if(save)	{end=true;text=*it;m_path=getPath(m_path)+text;}
				if (*it==text)
				{
					save=true;
				}
				else
				{
					save=false;
				}
			}
			++it;
			if(it==playList.end()) it = playList.begin();
		}
			int err=0;
			qDebug("NextTrack:"+text);
	}*/
#define Write(s) write(mfd_in, s, sizeof(s)-1)
void KPlayer::keyPressEvent(QKeyEvent *event)
	{
		paint=2;
		//printf("KEY: %x\n", event->key());
		int err = 0;
		switch (event->key())
		{
			case 0x1016: //	VolumeUp	(KEY_VOL_UP)
				err = write(mfd_in, "0", 1);//Write("0");
				volume=volume+3;
				paused = 0;
			break;
			
			case 0x1013: //	Up VolumeUp
				err = Write("0");
				volume=volume+3;
				paused = 0;
			break;
			
			case 0x1017: //	VolumeDown	(KEY_VOL_DOWN)
				err = Write("9");
				volume=volume-3;
				paused = 0;
			break;
			
			case 0x1015: //	Down VolumeDown
				err = Write("9");
				volume=volume-3;
				paused = 0;
			break;
			
			///
			
			case 0x1014: //	Right i.e. rotated Up	(KEY_RIGHT)
				err = Write("\033[C");
				paused = 0;
				prew_next=1;
			break;
			
			case 0x1012: //	Left i.e. rotated Down	(KEY_LEFT)
				err = Write("\033[D");
				paused = 0;
				prew_next=2;
			break;
			
			//
			
			case 4172: //	(KEY_NEXT)
				if(playlist)
					err = Write(">");
				else
					err = Write("\033[C");
				paused = 0;
				prew_next=1;
			break;
			
			case 0x103a: //	RightSoft -> PgUp
				err = Write(">");
				paused = 0;
				prew_next=1;
			break;
			
			case 4171: //	(KEY_PREW)
				if(playlist)
					err = Write("<");
				else
					err = Write("\033[D");
				paused = 0;
				prew_next=2;
			break;
			
			case 0x1038: //	LeftSoft -> PgDn
				err = Write("<");
				paused = 0;
				prew_next=2;
			break;
			
			///
			
			case 0x1004: //	Center -> Pause	(KEY_CENTR)
				err = Write("p");
				paused = !paused;
			break;
			
			case 4169: //	Center -> Pause	(KEY_PLAY)
				err = Write("p");
				paused = !paused;
			break;
			
			///
			
			case 35://	#	mute
				err = Write("m");
				mute=!mute;
				paused = 0;
			break;
			
			///
			
			case 0x1030: //	Send -> OSD menu [x]
				if (!paused)
				err = Write("osd");
			break;
			/*
			case 0x1048: //	Music -> Mute
				err = Write("m");
				paused = 0;
			break;
			
			case 0x1005: //	Internet
				err = Write("o");
				paused = 0;
			break;*/
			
			///
			/*case 48:
				qDebug("nextTrack");
				nextTrack();
			break;*/
			
			///
			
			case 0x1032: //	C [?]
				hide();
			break;
			
			case 0x1031: //	exit	(KEY_RED)
				timer->stop();
				if (mfd_in >= 0)
				{
					Write("quit");
					close(mfd_in);
					mfd_in = -1;
					sleep(1);
					if (pid > 0)
					kill(pid, SIGKILL);
					qApp->quit();
					return;
				}
			break;
		}    
		
		/*if (event->key() >= 0x20 && event->key() <= 0x7f)
		{
			unsigned char c = event->key();
			err = Write(&c);
			paused = 0;
		}*/
	}
	
void KPlayer::keyReleaseEvent(QKeyEvent *event)
	{
		switch (event->key())
		{
			default:
				paint=2;
				prew_next=0;
			break;
		}
	}
	
void KPlayer::focusOutEvent(QFocusEvent * )
	{
		/*if (!paused)
		{
			Write("p");
			paused = !paused;
		}*/
		paint=0;
		timer->start( 2000 );
		//KPlayer::focusOutEvent(event);
	}

void KPlayer::focusInEvent(QFocusEvent * )
	{
		qDebug("focusInEvent");
		/*if (paused)
		{
			Write("p");
			paused = !paused;
		}*/
		paint=2;
		timer->start( 40 );
		//KPlayer::focusInEvent(event);
	}

void KPlayer::drawAbout()
	{
		if(!ypoCkhc())	// check copyright
		{
			Write("quit");
			close(mfd_in);
			mfd_in = -1;
			sleep(1);
			if (pid > 0)
			kill(pid, SIGKILL);
			qApp->quit();
			return;
		}
		
		//
		
		p.begin( about_scr, this );
		p.setPen(Qt::black);
		p.setBrush(QColor(200, 200, 200));
		p.drawRect(0, 0, 220, 120);
		
		p.setPen(QColor(20, 20, 20));
		p.setBrush(QColor(40, 60, 130));
		p.drawRect(4, 29, 212, 87);
		
		p.setFont(QFont(NULL, 14, 25, false));
		p.setPen(QColor(220, 220, 220));		
		p.drawText(5, 40, 210, 20, Qt::AlignCenter, text2, -1);
		p.drawText(5, 60, 210, 20, Qt::AlignCenter, text3, -1);
		p.drawText(5, 90, 210, 20, Qt::AlignCenter, text4, -1);
		
		///	
		
		p.setPen(QColor(65, 65, 65));
		p.setBrush(QColor(30, 30, 30));
		p.drawRect(1, 1, 218, 25);
		
		p.setPen(QColor(130, 130, 130));
		p.drawLine(2, 2, 217, 2);
		int col=0;
		for (int i=3; i<=16; i++)
		{
			col=100-(i-3)*5;
			p.setPen(QColor(col, col, col));
			p.drawLine(2, i, 217, i);
		}
		
		p.setFont(QFont(NULL, 16, 87, false));
		p.setPen(QColor(220, 220, 220));		
		p.drawText(5, 5, 210, 20, Qt::AlignCenter, title, -1);
		
		p.end();
	}

bool KPlayer::ypoCkhc()
	{
		bool ret=true;
		
		title=QString("SVKPlayer 0.6");
		
		char txt2[35] =
		{
			(char)40,	(char)99,	(char)41,	(char)32,	(char)87,	(char)114,	(char)105,	(char)116,	(char)116,
			(char)101,	(char)110,	(char)32,	(char)98,	(char)121,	(char)32,	(char)83,	(char)116,	(char)114,
			(char)101,	(char)108,	(char)111,	(char)107,	(char)32,	(char)86,	(char)108,	(char)97,	(char)100,
			(char)32,	(char)75,	(char)111,	(char)108,	(char)101,	(char)114,	(char)116,	(char)115
		};
		
		char txt3[18] =
		{
			(char)97,	(char)100,	(char)109,	(char)105,	(char)110,	(char)64,	(char)109,	(char)111,	(char)98,
			(char)115,	(char)107,	(char)46,	(char)111,	(char)114,	(char)103,	(char)46,	(char)117,	(char)97
		};
		
		char txt4[17] =
		{
			(char)104,	(char)116,	(char)116,	(char)112,	(char)58,	(char)47,	(char)47,	(char)77,	(char)111,
			(char)116,	(char)111,	(char)70,	(char)97,	(char)110,	(char)46,	(char)82,	(char)85
		};
		
		for(int i=0; i<35; i++)
		{
			text2+=QString("%1").arg(txt2[i]);
		}
		
		for(int i=0; i<18; i++)
		{
			text3+=QString("%1").arg(txt3[i]);
		}
		
		for(int i=0; i<17; i++)
		{
			text4+=QString("%1").arg(txt4[i]);
		}
		
		//(c) Written by Strelok Vlad Kolerts
		
		if(title.mid( 2, 1 )!="K" && ret) ret=false;
		
		if(text2.mid( 18, 1 )!="e" && ret) ret=false;
		if(text2.mid( 25, 1 )!=QString("%1").arg((char)97) && ret) ret=false;
		if(text3.mid( 6, 1 )!="m" && ret) ret=false;
		if(text3.mid( 10, 1 )!=QString("%1").arg((char)107) && ret) ret=false;
		if(text4.mid( 15, 2 )!=QString("%1%2").arg((char)82).arg((char)85) && ret) ret=false;
		
		return ret;
	}
	
void KPlayer::readConfig()
	{
		path="/mmc/mmca1/.system/QTDownLoad/fm/";
		if (QFile(path).exists())
		{
			qDebug("FsMan install in SD");
			ZConfig fm_cfg(path+"fm.cfg", true);
			work_folder=fm_cfg.readEntry(QString("Setting"), QString("Work_folder"), "");
			qDebug("Work folder:"+work_folder);
			path=work_folder+"/plugins/SVKPlayer/";
			if (!QFile(path).exists())
			{
				qDebug("installed version of FsMan < 1.5");
				path="/mmc/mmca1/.system/QTDownLoad/fm/plugins/SVKPlayer/";
			}
		}
		else
		{
			path="/ezxlocal/.system/QTDownLoad/fm/";
			if (QFile(path).exists())
			{
				qDebug("FsMan install in Phone");
				ZConfig fm_cfg(path+"fm.cfg", true);
				work_folder=fm_cfg.readEntry(QString("Setting"), QString("Work_folder"), "");
				qDebug("Work folder:"+work_folder);
				path=work_folder+"/plugins/SVKPlayer/";
				if (!QFile(path).exists())
				{
					qDebug("installed version of FsMan < 1.5");
					path="/ezxlocal/.system/QTDownLoad/fm/plugins/SVKPlayer/";
				}
			}
			else
			{
				qDebug("ERORR! cant found FsMan");
				qApp->quit();
			}
		}
		ZConfig svk_cfg(path+"config.txt", true);
		
		skin_path=path+"/skin/"+svk_cfg.readEntry(QString("Setting"), QString("Current_skin"), "")+"/";
		mplayer=svk_cfg.readEntry(QString("Setting"), QString("MPlayer_path"), "");
		readTegs=atoi(svk_cfg.readEntry(QString("Setting"), QString("Read_Tegs"), ""));
		
		qDebug("Skin path:"+skin_path);
		qDebug("MPlayer path:"+mplayer);
	}
	
void KPlayer::readSkinConfig()
	{
		ZConfig skini(QString("%1skin.ini").arg(skin_path), true);
		pp_x=atoi(skini.readEntry(QString("Positions"),			QString("PlayPauseButton_X"), ""));
		pp_y=atoi(skini.readEntry(QString("Positions"),			QString("PlayPauseButton_Y"), ""));
		next_x=atoi(skini.readEntry(QString("Positions"),		QString("NextButton_X"), ""));
		next_y=atoi(skini.readEntry(QString("Positions"),		QString("NextButton_Y"), ""));
		prew_x=atoi(skini.readEntry(QString("Positions"),		QString("PrewButton_X"), ""));
		prew_y=atoi(skini.readEntry(QString("Positions"),		QString("PrewButton_Y"), ""));
		vol_p_x=atoi(skini.readEntry(QString("Positions"),		QString("VolumePic_X"), ""));
		vol_p_y=atoi(skini.readEntry(QString("Positions"),		QString("VolumePic_Y"), ""));
		vol_t_x=atoi(skini.readEntry(QString("Positions"),		QString("VolumeText_X"), ""));
		vol_t_y=atoi(skini.readEntry(QString("Positions"),		QString("VolumeText_Y"), ""));
		kbps_x=atoi(skini.readEntry(QString("Positions"),		QString("KbpsText_X"), ""));
		kbps_y=atoi(skini.readEntry(QString("Positions"),		QString("KbpsText_Y"), ""));
		khz_x=atoi(skini.readEntry(QString("Positions"),		QString("KhzText_X"), ""));
		khz_y=atoi(skini.readEntry(QString("Positions"),		QString("KhzText_Y"), ""));
		chanels_x=atoi(skini.readEntry(QString("Positions"),	QString("ChanelsText_X"), ""));
		chanels_y=atoi(skini.readEntry(QString("Positions"),	QString("ChanelsText_Y"), ""));
		s_name[0]=atoi(skini.readEntry(QString("Positions"),	QString("NameText_X"), ""));
		s_name[1]=atoi(skini.readEntry(QString("Positions"),	QString("NameText_Y"), ""));
		s_name[2]=atoi(skini.readEntry(QString("Positions"),	QString("NameText_W"), ""));
		s_name[3]=atoi(skini.readEntry(QString("Positions"),	QString("NameText_H"), ""));
		
		name_font_size=atoi(skini.readEntry(QString("Fonts"),	QString("NameFontSize"), ""));
		vol_font_size=atoi(skini.readEntry(QString("Fonts"),	QString("VolumeFontSize"), ""));
		detal_font_size=atoi(skini.readEntry(QString("Fonts"),	QString("DetailsFontSize"), ""));
		
		name_font_col=skini.readEntry(QString("Colors"),	QString("NameFontColor"), "");
		vol_font_col=skini.readEntry(QString("Colors"),	QString("VolumeFontColor"), "");
		detal_font_col=skini.readEntry(QString("Colors"),	QString("DetailsFontColor"), "");
	}
/*	
void KPlayer::PlayListGen()
	{
		QString	filter;
		QFile file(work_folder+"mime_types.cfg");
		if( file.open(IO_ReadOnly) )		
		{
			int i=0;
			QTextStream	stream( &file );
			QString		temp_line;
			while(temp_line=stream.readLine())
			{
				if(atoi(temp_line.mid(0,3))==3)	filter+="*."+temp_line.mid(7,10)+" ";
				i++;
			}
			file.close();	
		}
		
		QDir dir( getPath(m_path), filter);
		dir.setMatchAllDirs ( false );
		if ( !dir.isReadable() )
		return;
		playList = dir.entryList();
		playList.sort();
	}*/
	
void KPlayer::readMP3tegs()
	{
		if(!playlist && readTegs)
		{
			int err = 0;
			bool tegs_ok=false;
			QString hr;
			
			err = Write("p");
			
			mp3teg* tegs = new mp3teg(m_path);
			if(tegs->readArtist()!=NULL)
			{
				tegArtist=ANSI(tegs->readArtist());
				tegs_ok=true;
			}
			else
				tegArtist="Unknown";
			qDebug("tegArtist:"+tegArtist);
			
			if(tegs->readTitle()!=NULL)
			{
				tegTitle=ANSI(tegs->readTitle());
				tegs_ok=true;
				hr=" - ";
			}
			qDebug("tegTitle:"+tegTitle);
			
			tegAlbum=ANSI(tegs->readAlbum());
			tegYear=tegs->readYear();
			tegGenre=tegs->readGenre();
			tegComment=ANSI(tegs->readComment());
			
			tegBitrate=tegs->readBitRate();
			tegHz=tegs->readHz();
			tegChanels=tegs->readChanels();
			
			if(tegs_ok)
				m_name=tegArtist+hr+tegTitle;
			
			err = Write("p");
		}
		qDebug("m_name:"+m_name);
		
		QFontMetrics fm = fontMetrics();
		leight_name=fm.width(m_name);
		
		
		paint=2;
	}

QString KPlayer::getPath ( const QString &AFileName )
{
  QString str = AFileName;
  int i = str.findRev ( "/" );
  str.remove ( i+1, str.length() - i );
  return str;
}

QString KPlayer::getFileName(const QString &AFileName)
{
	QString str = AFileName;
	int i = str.findRev("/");
	str.remove(0, i);
	if (str.left(1) = "/") str.remove(0, 1);
	return str;
}
	
QString getMime(QString file_name)
{
	int i = file_name.findRev (".");
	return file_name.mid(i+1,file_name.length()-i);
}

void KPlayer::animate()
	{
		if(about_c>0)	{about=true; about_c--;}
		else			{about=false;resize(240, p_bg_height);move(0,320-p_bg_height);/*hide();QTimer::singleShot(100, this, SLOT(show()));*/}
		
		if(about_c==18)
			readMP3tegs();
		
		if(volume>100) volume=100;
		if(volume<1) volume=0;
		
		if(about)	paintAbout();
		else
		{
			if(paint==2)	paintPlayer();
			if(paint>0)	paintScrollText();
		}
		int callstat=UTIL_GetIncomingCallStatus();
		//qDebug(QString("callstat=%1").arg(callstat));
		
		int err = 0;
		err = write(mfd_in, "", 1);
		if (err < 0 or callstat==1) 
		{
			if (pid > 0)
				kill(pid, SIGKILL);
			timer->stop();
			qApp->quit();
		}
		
		/*char buf[16384]={0};
		int r = read(mfd_out,buf,sizeof(buf));
		qDebug(QString("Read %1 '%2'").arg(r).arg(buf));*/
		
	}
	
void KPlayer::slotQCopReceived(const QCString &msg, const QByteArray &data)
{
	paint=2;
	int err = 0;
	
	if (msg == "show") show();
	if (msg == "play/pause")
	{
		err = Write("p");
		paused = !paused;
	}
	if (msg == "next")
	{
		err = Write(">");
		paused = !paused;
	}
	if (msg == "prew")
	{
		err = Write("<");
		paused = !paused;
	}
	
	if (msg=="keyMsg(int,int)")
	{
		QDataStream stream(data, IO_ReadOnly);
		int key, type;
		stream >> key >> type;
		if(type==1)
		{
			if(key==4169)
			{
				show();
			}
			if(key==4172)
			{
				err = Write(">");
				paused = !paused;
			}
			if(key==4171)
			{
				err = Write("<");
				paused = !paused;
			}
		}
	}
}
/*
void KPlayer::slotHardKeyQCopReceived(const QCString &msg, const QByteArray &data)
{
	paint=2;
	int err = 0;
	
	qDebug(QString("%1").arg(msg));
	
	if (msg=="keyMsg(int,int)")
	{
		QDataStream stream(data, IO_ReadOnly);
		int key, type;
		stream >> key >> type;
		qDebug(QString("key:%1 type:%2").arg(key).arg(type));
		if(type==1)
		{
			if(key==4169)
			{
				show();
			}
			if(key==4171)
			{
				err = Write(">");
				paused = !paused;
			}
			if(key==4172)
			{
				err = Write("<");
				paused = !paused;
			}
		}
	}
}*/

int main(int argc, char **argv)
{
    ZApplication* a = new ZApplication(argc, argv);
    char * mfile;
	playlist=false;

    mfile = argv[2];
    if (mfile == NULL)
	{
		qApp->quit();
		delete a;
		return 0;
    }
	
	if(getMime(mfile)=="m3u" or getMime(mfile)=="pls")
		playlist=true;
	
	
    KPlayer* dlg = new KPlayer(mfile);
    a->setMainWidget(dlg);
	
	QString mplayer=dlg->getMPlayerPath();
	QString home=dlg->getHomePath();
	
    dlg->show();
    
	
    /*char ** margs = (char**)alloca(sizeof(char*)*(4 + 0 + argc));
    int i = 0;
    margs[i++] = "mplayer";
    margs[i++] = "-slave";
    margs[i++] = mfile;*/
	
	
	int pfd[2];
    pipe(pfd);
    pid = vfork();
    if (pid == 0)
	{
		int i;
		dup2(pfd[0], 0);
		for (i=3; i<256; i++)
			close(i);
		setenv("HOME", home, 1);
		
		if(playlist) execl(mplayer, "mplayer", "-playlist", mfile, NULL);
		else execl(mplayer, "mplayer", mfile, NULL);
		//execv(mplayer, margs);
		_exit(127);
    }
    close(pfd[0]);
	//mfd_out = pfd[0];
    mfd_in = pfd[1];
    signal(SIGPIPE, SIG_IGN);
    a->exec();
	
    return 0;
}


#include "SVKPlayer.moc"
